#include "TestEnvTests.h"
#include "TestEnv/Testing.h"
#include "CoralBase/Exception.h"
#include <iostream>
#include <exception>
#include <cstdlib>

int main(int argc, char *argv[])
{
  TestEnvTests app("LOOKUPSERVICE");
  
  app.addWriters().addDefault("sqlite").add("mysql").add("oracle");

  if(app.check(argc, argv)){
  
    //default write access to the database
    app.addServiceName(0, TEST_CORE_SCHEME_ADMIN);
    
    app.addServiceName(0, TEST_CORE_SCHEME_READER);  
    
    app.addServiceName(0, TEST_CORE_SERVICE_0);
    app.addServiceName(0, TEST_CORE_SERVICE_1);
    app.addServiceName(0, TEST_CORE_SERVICE_2);
    app.addServiceName(0, TEST_CORE_SERVICE_3);
    
    app.addServiceName(0, TEST_CORE_REPLICA_SERVICE_0);
    app.addServiceName(0, TEST_CORE_REPLICA_SERVICE_1);
    app.addServiceName(0, TEST_CORE_REPLICA_SERVICE_2);
    app.addServiceName(0, TEST_CORE_REPLICA_SERVICE_3);
    app.addServiceName(0, TEST_CORE_REPLICA_SERVICE_4);
    
    app.addServiceName(0, TEST_CORE_REPLICA_EXCLUDING_0);
    app.addServiceName(0, TEST_CORE_REPLICA_EXCLUDING_1);
    app.addServiceName(0, TEST_CORE_REPLICA_EXCLUDING_2);
    
    app.addServiceName(0, TEST_CORE_DEVSERVICE_0);
    app.addServiceName(0, TEST_CORE_DEVSERVICE_1);    
    //a failover service name with two update entries
    app.addServiceName(0, TEXT_CORE_FAILOVER_SERVICE_0);
    
    try {
    
      //test first if we get the correct replica number for the service name
      //admin should have one entry -> one replicas
      app.testReplicas(0, 1, 1);
      
      app.testReplicas(1, 0, 1);
      
      app.testReplicas(2, 1, 1);
      app.testReplicas(3, 1, 1);
      app.testReplicas(4, 1, 1);
      app.testReplicas(5, 1, 1);
      
      app.testReplicas(6, 0, 0);
      app.testReplicas(7, 1, 1);
      app.testReplicas(8, 1, 1);
      app.testReplicas(9, 2, 4);
      app.testReplicas(10, 2, 2);
      
      app.testReplicas(11, 2, 2);
      app.testReplicas(12, 0, 2);
      app.testReplicas(13, 2, 2);
      
      app.testReplicas(14, 1, 2);
      app.testReplicas(15, 1, 2);
      
      app.testReplicas(16, 0, 2);
    }
    catch ( coral::Exception& e )
    {
      std::cerr << "CORAL exception " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e )
    {
      std::cerr << "Standard C++ exception " << e.what() << std::endl;
      return 1;
    }
    catch( ... )
    {
      std::cerr << "Unknown exception ..." << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Success" << std::endl;
    return 0;
  }
  return 1;
}
