#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "RandomQuery.h"

#include "CoralBase/Exception.h"


#include "TestEnv/TestEnv.h"

#include <iostream>

static void * testthread(void * param)
{
  TestEnv * TC01 = static_cast<TestEnv*>(param);
  RandomQuery rq(TC01);
  
  rq.createSession(0, TESTENV_READER, false);
  
  for(size_t i = 0; i < 100; i++){
    sleep((rand() % 3));
    rq.execute();
  }
  
  rq.deleteSession();
  
  return NULL;
}
      
#define __RANDOM_CYCLER_THREADS 10

int main(int argc, char *argv[]){

  TestEnv TC01("RANDOM");
  
  TestEnvWriters& wr01 = TC01.addWriters().addDefault("oracle").add("mysql").add("sqlite");
  wr01.readers("oracle").addDefault("oracle").add("coral");  
  wr01.readers("mysql").addDefault("mysql");
  wr01.readers("sqlite").addDefault("sqlite").add("coral");
  
  if(TC01.check(argc, argv)){
  
    //add the default connection strings to the test application
    TC01.addServiceName(0, TEST_CORE_SCHEME_ADMIN, TEST_CORE_SCHEME_READER);
 
    try {
      RandomQuery rq(&TC01);
      rq.createSession(0);
      rq.setup();
      rq.deleteSession();
      
  
      pthread_t id[__RANDOM_CYCLER_THREADS];
      pthread_attr_t attr;
      
      pthread_attr_init(&attr);
      pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
    
      for( size_t i = 1; i < __RANDOM_CYCLER_THREADS; i++ ){
        pthread_create(&id[i], &attr, testthread, (void*)&TC01);
      }
  
      void *status;
      for( size_t i = 1; i < __RANDOM_CYCLER_THREADS; i++ ){
        pthread_join(id[i], &status);
      }

            

    }
    catch ( coral::Exception& e ) {
      std::cerr << "CORAL Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e ) {
      std::cerr << "C++ Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( ... ) {
      std::cerr << "Unhandled exception " << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Success" << std::endl;
      
    return 0;
  }
  return 1;
}
